-- top level vhdl file. original code from xess.com
--
-- NTSC stuff
-- jun 27 2005
-- july 1 2005 more work...
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use WORK.common.all;
use WORK.xilinx.all;
use WORK.this_dont_matter.all;
use WORK.sdram.all;

-- synchronous DRAM tester (originally from the sdram tester - www.xess.com)

entity sdramTst is
	generic(
		DATA_WIDTH: natural := 16;				-- SDRAM data width
		BEG_ADDR: natural := 0;						-- beginning SDRAM address
		END_ADDR: natural := 4_194_303;		-- ending SDRAM address
		BEG_TEST: natural := 0;						-- beginning test range address
		END_TEST: natural := 4_194_303		-- ending test range address
	);
	port(
		ce_n: out std_logic;			-- Flash RAM chip-enable
		rst_n: in std_logic;			-- main system reset input
		clkin: in std_logic;			-- main clock input from external clock source
		sclkfb: in std_logic;			-- feedback SDRAM clock with PCB delays
		sclk: out std_logic;			-- clock to SDRAM
		cke: out std_logic;				-- SDRAM clock-enable
		cs_n: out std_logic;			-- SDRAM chip-select
		ras_n: out std_logic;			-- SDRAM RAS
		cas_n: out std_logic;			-- SDRAM CAS
		we_n: out std_logic;			-- SDRAM write-enable
		ba: out unsigned( 1 downto 0);		-- SDRAM bank-address
		sAddr: out unsigned(11 downto 0);	-- SDRAM address bus
		sData: inout unsigned(DATA_WIDTH-1 downto 0);	-- data bus to SDRAM
		dqmh: out std_logic;					-- SDRAM DQMH
		dqml: out std_logic;					-- SDRAM DQML
		s: out unsigned(6 downto 0);			-- 7-segment LED
		pps: out std_logic_vector(6 downto 3);	-- outputs to parallel port status bits

		-- NTSC signals
		reset: in std_logic;
		data: out unsigned (7 downto 0)
		--debug_line: out unsigned (DATA_WIDTH-1 downto 0)
	);

end sdramTst;

architecture arch of sdramTst is

	component IBUF port (I : in STD_LOGIC; O : out STD_LOGIC); end component;
	--XILINX DEFINITON
	component IBUFG port (I : in STD_LOGIC; O : out STD_LOGIC); end component;
	component OBUF port (I : in STD_LOGIC; O : out STD_LOGIC); end component;
	--XILINX DEFINITION
	component BUFG port (I : in STD_LOGIC; O : out STD_LOGIC); end component;
	--XILINX DEFINITION
	component BUFGP port (I: in std_logic; o: out std_logic); end component;
component clocks 
	port(
		clk: in std_logic; 
		reset : in std_logic;
		--clk_out: out std_logic;
		clk_div_4: out std_logic
		--clk_for_kb_sampling: out std_logic
		);
end component;


	constant ADDR_WIDTH: natural := log2(END_ADDR-BEG_ADDR+1);
	signal  tmp, pclk_buffered: std_logic;
	signal clk_for_kb_sampling: std_logic;
	signal clock_div_4: std_logic;
	signal slow_clock: std_logic;
	signal reset_after_bufg: std_logic;
	signal rst_int: std_logic;		-- internal reset signal
	signal clk_int: std_logic;		-- internal master clock signal
	signal bufclkin: std_logic;		-- buffered input (non-DLL) clock
	signal lock: std_logic; 		-- SDRAM clock DLL lock indicator
	signal done: std_logic;			-- SDRAM operation complete indicator
	signal hAddr: unsigned(ADDR_WIDTH-1 downto 0);		-- host address bus
	signal hDIn: unsigned(DATA_WIDTH-1 downto 0);			-- host-side data to SDRAM
	signal hDOut: unsigned(DATA_WIDTH-1 downto 0);		-- host-side data from SDRAM
	signal rd: std_logic;															-- host-side read control signal
	signal wr: std_logic;															-- host-side write control signal
	signal dataIn: unsigned(DATA_WIDTH-1 downto 0);		-- input databus from SDRAM
	signal dataOut: unsigned(DATA_WIDTH-1 downto 0);	-- output databus to SDRAM
	signal divCnt: unsigned(22 downto 0);							-- clock divider
	signal progress: std_logic_vector(1 downto 0);		-- test progres indicator
	signal err: std_logic;														-- test error flag
	signal sdramCntl_state: std_logic_vector(3 downto 0);	-- SDRAM controller state
begin

	ce_n <= '1';	-- disable Flash RAM

	-- synchronous reset.  internal reset flag is set active by config. bitstream
	-- and then gets reset after DLL clocks start.
	process(bufclkin)
	begin
		if(bufclkin'event and bufclkin='1') then
			if lock='0' then
				rst_int <= '1';				-- keep in reset until DLLs start up
			else
				rst_int <= not rst_n;	-- else manually activate reset with pushbutton
			end if;
		end if;
	end process;

	-- generic memory tester module
	--u0: memTest
	u0: ntsc
	generic map(
		DATA_WIDTH => hDIn'length,
		ADDR_WIDTH => hAddr'length,
		BEG_TEST => BEG_TEST,
		END_TEST => END_TEST		
	)
	port map(
		clk => clk_int,				-- master internal clock
		---clk=>clkin,  --- ????QUESTION????
		rst => rst_int,				-- reset
		ack => done,					-- SDRAM controller operation complete
		dIn => hDOut,					-- host-side data from SDRAM goes to memory tester
		rd => rd,							-- host-side SDRAM read control from memory tester
		wr => wr,							-- host-side SDRAM write control from memory tester
		addr => hAddr,				-- host-side address from memory tester
		dOut => hDIn,					-- host-side data to SDRAM comes from memory tester
		progress => progress,	-- current phase of memory test
		err => err,					-- memory error flag
	    reset => '1',  -- horrible hack to jump start the NTSC state machine
	    -- REMOVED FROM ORIGINAL clk: in std_logic;
		--ntsc_clk=>clkin, --- CHANGED
		ntsc_clk=>clk_int,
		data =>data
		--debug_line=>debug_line
	   
	);

	pclk_buffered<= not clkin;
	-- SDRAM memory controller module
	 u1: sdramCntl
	 generic map(
     FREQ => 100_000,     -- 100 MHz operation
		 DATA_WIDTH => hDIn'length,
		 NROWS => 4096,
		 NCOLS => 256,
		 HADDR_WIDTH => hAddr'length,
		 SADDR_WIDTH => sAddr'length
	 )
	 port map(
		 --clkin => clkin, -- master clock from external clock source(unbuffered)
		clkin=>clkin,
		 bufclk => bufclkin,	-- buffered master clock output
		 clk0 => clk_int,		-- synchronized master clock (accounts for delays to external SDRAM)
		 clk2x => open,			-- synchronized doubled master clock
		 lock => lock,				-- valid synchronized clocks indicator
		 rst => rst_int,			-- reset
		 rd => rd,						-- host-side SDRAM read control from memory tester
		 wr => wr,						-- host-side SDRAM write control from memory tester
		 done => done,					-- SDRAM memory read/write done indicator
		 hAddr => hAddr,				-- host-side address from memory tester
		 hDIn => hDIn,					-- test data pattern from memory tester
		 hDOut => hDOut,				-- SDRAM data output to memory tester
		 sdramCntl_state => sdramCntl_state,	-- SDRAM controller state (for diagnostics)
		 sclkfb => sclkfb,			-- clock feedback with added external PCB delays
		 sclk => sclk,					-- synchronized clock to external SDRAM
		 cke => cke,						-- SDRAM clock enable
		 cs_n => cs_n,					-- SDRAM chip-select
		 ras_n => ras_n,				-- SDRAM RAS
		 cas_n => cas_n,				-- SDRAM CAS
		 we_n => we_n,					-- SDRAM write-enable
		 ba => ba,						-- SDRAM bank address
		 sAddr => sAddr,				-- SDRAM address
		 sData => sData,				-- SDRAM databus
		 dqmh => dqmh,					-- SDRAM DQMH
		 dqml => dqml					-- SDRAM DQML
	 );

	-- indicate the phase of the memory test on the LED	
	s <=	"0010010" when progress="00" else		-- "1"
				"1011101" when progress="01" else	-- "2"
				"1011011" when progress="10" else	-- "3"
				"1101101" when err=YES else			-- "E" when error occurs
				"1110111";							-- "O" when memory tests OK

	-- generate some slow signals from the master clock
	process(bufclkin)
	begin
		if(bufclkin'event and bufclkin='1') then
			divCnt <= divCnt+1;
		end if;
	end process;

	-- send a heartbeat signal back to the PC so the testing program knows
	-- the status of the memory test				
	pps(6) <= divCnt(20) when progress/="11"	else -- slow clock means test in progress
				divCnt(18) when err=NO else		-- faster clock indicates a memory error
						'0';					-- no clock means test passed

end arch;
